from django.urls import reverse

from django.db import models
from django.db.models import F, Q, CheckConstraint
from django.core.exceptions import ValidationError

from ipam.fields import IPNetworkField
from netbox.models import NetBoxModel
from netbox_routing.choices.objects import PermitDenyChoices


__all__ = (
    'RouteMap',
    'RouteMapEntry',
    'PrefixList',
    'PrefixListEntry'
)


class RouteMap(NetBoxModel):
    name = models.CharField(
        max_length=255
    )

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemap', args=[self.pk])

class RouteMapEntry(NetBoxModel):
    route_map = models.ForeignKey(
        to="netbox_routing.RouteMap",
        on_delete=models.PROTECT,
        related_name='entries',
        verbose_name='Route Map'
    )
    type = models.CharField(max_length=6, choices=PermitDenyChoices)
    sequence = models.PositiveSmallIntegerField()

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:routemapentry', args=[self.pk])


class PrefixList(NetBoxModel):
    name = models.CharField(
        max_length=255
    )

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlist', args=[self.pk])


class PrefixListEntry(NetBoxModel):
    prefix_list = models.ForeignKey(
        to="netbox_routing.PrefixList",
        on_delete=models.PROTECT,
        related_name='entries',
        verbose_name='Prefix List'
    )
    sequence = models.PositiveSmallIntegerField()
    type = models.CharField(max_length=6, choices=PermitDenyChoices)
    prefix = IPNetworkField(help_text='IPv4 or IPv6 network with mask')
    ge = models.PositiveSmallIntegerField()
    le = models.PositiveSmallIntegerField()

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:prefixlistentry', args=[self.pk])

    def clean(self):
        super().clean()

        if self.prefix.version == 6:
            if self.le is not None and self.le > 128:
                raise ValidationError({
                    'le': 'LE value cannot be longer then 128'
                })
            if self.ge is not None and self.ge > 128:
                raise ValidationError({
                    'ge': 'GE value cannot be longer then 128'
                })
        elif self.prefix.version == 4:
            if self.le is not None and self.le > 32:
                raise ValidationError({
                    'le': 'LE value cannot be longer then 32'
                })
            if self.ge is not None and self.ge > 32:
                raise ValidationError({
                    'ge': 'GE value cannot be longer then 32'
                })

        if self.ge and self.le and self.ge < self.le:
            raise ValidationError({
                    'ge': 'GE cannot be more then LE',
                    'le': 'LE cannot be less then GE'
            })

        if self.ge is not None and self.prefix.prefix.prefixlen >= self.ge:
            raise ValidationError('Prefix\'s length cannot be longer then greater or equals value')

        if self.le is not None and self.prefix.prefix.prefixlen >= self.le:
            raise ValidationError('Prefix\'s length cannot be longer then greater or equals value')





from netbox.views.generic import ObjectListView, ObjectView, ObjectEditView, ObjectDeleteView
from netbox_routing.filtersets import PrefixListFilterSet, PrefixListEntryFilterSet, RouteMapEntryFilterSet, \
    RouteMapFilterSet
from netbox_routing.forms import PrefixListFilterSetForm, PrefixListForm, PrefixListEntryFilterSetForm, \
    PrefixListEntryForm, RouteMapEntryForm, RouteMapEntryFilterSetForm, RouteMapForm, RouteMapFilterSetForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMapEntry, RouteMap
from netbox_routing.tables.objects import PrefixListTable, PrefixListEntryTable, RouteMapEntryTable, RouteMapTable


#
# Prefix List
#
class PrefixListListView(ObjectListView):
    queryset = PrefixList.objects.all()
    table = PrefixListTable
    filterset = PrefixListFilterSet
    filterset_form = PrefixListFilterSetForm


class PrefixListView(ObjectView):
    queryset = PrefixList.objects.all()
    template_name = 'netbox_routing/prefixlist.html'


class PrefixListEditView(ObjectEditView):
    queryset = PrefixList.objects.all()
    form = PrefixListForm


class PrefixListDeleteView(ObjectDeleteView):
    pass


#
# Prefix List Entry
#
class PrefixListEntryListView(ObjectListView):
    queryset = PrefixListEntry.objects.all()
    table = PrefixListEntryTable
    filterset = PrefixListEntryFilterSet
    filterset_form = PrefixListEntryFilterSetForm


class PrefixListEntryView(ObjectView):
    queryset = PrefixListEntry.objects.all()
    template_name = 'netbox_routing/prefixlist.html'


class PrefixListEntryEditView(ObjectEditView):
    queryset = PrefixListEntry.objects.all()
    form = PrefixListEntryForm


class PrefixListEntryDeleteView(ObjectDeleteView):
    pass


#
# Route Map
#
class RouteMapListView(ObjectListView):
    queryset = RouteMap.objects.all()
    table = RouteMapTable
    filterset = RouteMapFilterSet
    filterset_form = RouteMapFilterSetForm


class RouteMapView(ObjectView):
    queryset = RouteMap.objects.all()
    template_name = 'netbox_routing/routemap.html'


class RouteMapEditView(ObjectEditView):
    queryset = RouteMap.objects.all()
    form = RouteMapForm


class RouteMapDeleteView(ObjectDeleteView):
    pass


#
# Prefix List Entry
#
class RouteMapEntryListView(ObjectListView):
    queryset = RouteMapEntry.objects.all()
    table = RouteMapEntryTable
    filterset = RouteMapEntryFilterSet
    filterset_form = RouteMapEntryFilterSetForm


class RouteMapEntryView(ObjectView):
    queryset = RouteMapEntry.objects.all()
    template_name = 'netbox_routing/prefixlist.html'


class RouteMapEntryEditView(ObjectEditView):
    queryset = RouteMapEntry.objects.all()
    form = RouteMapEntryForm


class RouteMapEntryDeleteView(ObjectDeleteView):
    pass



import django_filters
import netaddr

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.models import StaticRoute


class StaticRouteFilterSet(NetBoxModelFilterSet):

    prefix = django_filters.CharFilter(
        method='filter_prefix',
        label='Prefix',
    )

    next_hop = django_filters.CharFilter(
        method='filter_address',
        label='Prefix',
    )

    class Meta:
        model = StaticRoute
        fields = ('vrf', 'prefix', 'devices', 'metric', 'next_hop')

    def filter_prefix(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPNetwork(value).cidr)
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()

    def filter_address(self, queryset, name, value):
        if not value.strip():
            return queryset
        try:
            query = str(netaddr.IPAddress(value))
            return queryset.filter(prefix=query)
        except (netaddr.AddrFormatError, ValueError):
            return queryset.none()



from django.db.models import Q

BGPSETTING_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter') |
    Q(app_label='netbox_routing', model='bgpscope')
)

BGPAF_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter') |
    Q(app_label='netbox_routing', model='bgpscope') |
    Q(app_label='netbox_routing', model='bgpneighbor') |
    Q(app_label='ipam', model='VRF')
)

BGPPEER_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgprouter') |
    Q(app_label='netbox_routing', model='bgpscope') |
    Q(app_label='netbox_routing', model='bgpaddressfamily')
)

BGPPEERAF_ASSIGNMENT_MODELS = Q(
    Q(app_label='netbox_routing', model='bgppeer') |
    Q(app_label='netbox_routing', model='bgppeergroup') |
    Q(app_label='netbox_routing', model='bgptemplatepeer') |
    Q(app_label='netbox_routing', model='bgptemplatepeerpolicy')
)


from netbox.tables import NetBoxTable
from netbox_routing.models import StaticRoute


class StaticRouteTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = StaticRoute
        fields = ('pk', 'id', 'vrf', 'prefix', 'next_hop', 'name')
        default_columns = ('pk', 'id', 'vrf', 'prefix', 'next_hop', 'name')



from rest_framework import serializers

from netbox_routing.api.nested_serializers import NestedPrefixListSerializer, NestedRouteMapSerializer
from netbox.api.serializers import NetBoxModelSerializer
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


__all__ = (
    'StaticRouteSerializer'
)


class PrefixListSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')

    class Meta:
        model = PrefixList
        fields = ('url', 'id', 'name')


class PrefixListEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlistentry-detail')
    prefix_list = NestedPrefixListSerializer()

    class Meta:
        model = PrefixListEntry
        fields = ('url', 'id', 'prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')


class RouteMapSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlist-detail')

    class Meta:
        model = RouteMap
        fields = ('url', 'id', 'name')


class RouteMapEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(view_name='plugins-api:netbox_routing-api:prefixlistentry-detail')
    route_map = NestedRouteMapSerializer()


    class Meta:
        model = RouteMapEntry
        fields = ('url', 'id', 'route_map', 'sequence', 'type')



from .static import StaticRouteFilterSetForm
from .objects import PrefixListFilterSetForm, PrefixListEntryFilterSetForm, RouteMapFilterSetForm,\
    RouteMapEntryFilterSetForm

__all__ = (
    'StaticRouteFilterSetForm',
    'PrefixListFilterSetForm',
    'PrefixListEntryFilterSetForm',
    'RouteMapFilterSetForm',
    'RouteMapEntryFilterSetForm'
)



from dcim.models import Device
from dcim.tables import DeviceTable
from netbox.views.generic import ObjectListView, ObjectEditView, ObjectView, ObjectDeleteView
from netbox_routing.filtersets.static import StaticRouteFilterSet
from netbox_routing.forms import StaticRouteForm
from netbox_routing.forms.filtersets.static import StaticRouteFilterSetForm
from netbox_routing.models import StaticRoute
from netbox_routing.tables.static import StaticRouteTable


__all__ = (
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteEditView',
    'StaticRouteDeleteView',
)


class StaticRouteListView(ObjectListView):
    queryset = StaticRoute.objects.all()
    table = StaticRouteTable
    filterset = StaticRouteFilterSet
    filterset_form = StaticRouteFilterSetForm


class StaticRouteView(ObjectView):
    queryset = StaticRoute.objects.all()
    template_name = 'netbox_routing/staticroute.html'

    def get_extra_context(self, request, instance):
        devices = instance.devices.all()

        devices_table = DeviceTable(
            list(devices),
            exclude=('tenant', 'device_role', 'serial', 'asset_tag', 'face', 'primary_ip', 'airflow', 'primary_ip4',
            'primary_ip6', 'cluster', 'virtual_chassis', 'vc_position', 'vc_priority', 'comments', 'contacts', 'tags',
            'created', 'last_updated',),
            orderable=False
        )

        return {
            'devices': devices_table,
        }


class StaticRouteEditView(ObjectEditView):
    queryset = StaticRoute.objects.all()
    form = StaticRouteForm


class StaticRouteDeleteView(ObjectDeleteView):
    pass



from netbox.api.routers import NetBoxRouter
from .views import StaticRouteViewSet, PrefixListViewSet, RouteMapViewSet, PrefixListEntryViewSet, RouteMapEntryViewSet

router = NetBoxRouter()
router.register('staticroute', StaticRouteViewSet)
router.register('prefix-list', PrefixListViewSet)
router.register('prefix-list-entry', PrefixListEntryViewSet)
router.register('route-map', RouteMapViewSet)
router.register('route-map-entry', RouteMapEntryViewSet)
urlpatterns = router.urls



from netbox.forms import NetBoxModelFilterSetForm
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListFilterSetForm(NetBoxModelFilterSetForm):
    model = PrefixList


class PrefixListEntryFilterSetForm(NetBoxModelFilterSetForm):
    model = PrefixListEntry


class RouteMapFilterSetForm(NetBoxModelFilterSetForm):
    model = RouteMap


class RouteMapEntryFilterSetForm(NetBoxModelFilterSetForm):
    model = RouteMapEntry


from netbox.api.viewsets import ModelViewSet
from netbox_routing.api.serializers import PrefixListSerializer, PrefixListEntrySerializer, RouteMapSerializer, \
    RouteMapEntrySerializer
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListViewSet(ModelViewSet):
    queryset = PrefixList.objects.all()
    serializer_class = PrefixListSerializer


class PrefixListEntryViewSet(ModelViewSet):
    queryset = PrefixListEntry.objects.all()
    serializer_class = PrefixListEntrySerializer


class RouteMapViewSet(ModelViewSet):
    queryset = RouteMap.objects.all()
    serializer_class = RouteMapSerializer


class RouteMapEntryViewSet(ModelViewSet):
    queryset = RouteMapEntry.objects.all()
    serializer_class = RouteMapEntrySerializer



from netbox.api.viewsets import ModelViewSet
from netbox_routing.api.serializers import StaticRouteSerializer
from netbox_routing.models import StaticRoute


class StaticRouteViewSet(ModelViewSet):
    queryset = StaticRoute.objects.all()
    serializer_class = StaticRouteSerializer



from utilities.choices import ChoiceSet


class BGPAdditionalPathSelectChoices(ChoiceSet):
    ALL = 'all'
    BACKUP = 'backup'
    BEST_EXTERNAL = 'best-external'
    GROUP_BEST = 'group-best'

    CHOICES = [
        (ALL, 'All'),
        (BACKUP, 'Backup'),
        (BEST_EXTERNAL, 'Best External'),
        (GROUP_BEST, 'Group Best')
    ]


class BGPBestPathASPath(ChoiceSet):
    IGNORE = 'ignore'
    MULTIPATH = 'multipath-relax'

    CHOICES = [
        (IGNORE, 'Ignore'),
        (MULTIPATH, 'Multipath Relax Comparison')
    ]


class BGPAddressFamilies(ChoiceSet):
    IPV4_UNICAST = 'ipv4-unicast'
    IPV6_UNICAST = 'ipv6-unicast'
    VPNV4_UNICAST = 'vpnv4-unicast'
    VPNV6_UNICAST = 'vpnv6-unicast'
    IPV4_MULTICAST = 'ipv4-multicast'
    IPV6_MULTICAST = 'ipv6-multicast'
    VPNV4_MULTICAST = 'vpnv4-multicast'
    VPNV6_MULTICAST = 'vpnv6-multicast'
    IPV4_FLOWSPEC = 'ipv4-flowspec'
    IPV6_FLOWSPEC = 'ipv6-flowspec'
    VPNV4_FLOWSPEC = 'vpnv4-flowspec'
    VPNV6_FLOWSPEC = 'vpnv6-flowspec'
    NSAP = 'nsap'
    L2VPNVPLS = 'l2vpn-vpls'
    L2VPSEVPN = 'l2vpn-evpn'
    LINKSTATE = 'link-state'
    RTFILTER_UNICAST = 'rtfilter-unicast'

    CHOICES = [
        (IPV4_UNICAST, 'IPv4 Unicast'),
        (IPV6_UNICAST, 'IPv6 Unicast'),
        (VPNV4_UNICAST, 'VPNv4 Unicast'),
        (VPNV6_UNICAST, 'VPNv6 Unicast'),
        (IPV4_MULTICAST, 'IPv4 Multicast'),
        (IPV6_MULTICAST, 'IPv6 Multicast'),
        (VPNV4_UNICAST, 'VPNv4 Multicast'),
        (VPNV6_MULTICAST, 'VPNv6 Multicast'),
        (IPV4_FLOWSPEC, 'IPv4 Flowspec'),
        (IPV6_FLOWSPEC, 'IPv6 Flowspec'),
        (VPNV4_FLOWSPEC, 'VPNv4 Flowspec'),
        (VPNV6_FLOWSPEC, 'VPNv6 Flowspec'),
        (NSAP, 'NSAP'),
        (L2VPNVPLS, 'L2VPN VPLS'),
        (L2VPSEVPN, 'L2VPN EVPN'),
        (LINKSTATE, 'LINK-STATE'),
        (RTFILTER_UNICAST, 'RTFILTER')
    ]



from .filtersets import *
from .objects import PrefixListForm, PrefixListEntryForm, RouteMapForm, RouteMapEntryForm
from .static import StaticRouteForm

__all__ = (
    # Static Routes
    'StaticRouteForm',
    'StaticRouteFilterSetForm',

    # Objects
    'PrefixListForm',
    'PrefixListEntryForm',
    'RouteMapForm',
    'RouteMapEntryForm',
    'PrefixListFilterSetForm',
    'PrefixListEntryFilterSetForm',
    'RouteMapFilterSetForm',
    'RouteMapEntryFilterSetForm'
)



from .static import StaticRoute
from .objects import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry

__all__ = (
    'StaticRoute',
    'PrefixList',
    'PrefixListEntry',
    'RouteMap',
    'RouteMapEntry'
)



from .static import StaticRouteListView, StaticRouteEditView, StaticRouteView, StaticRouteDeleteView

from .objects import PrefixListView, PrefixListEditView, PrefixListListView, PrefixListDeleteView, RouteMapListView, \
    RouteMapView, RouteMapEditView, RouteMapDeleteView, PrefixListEntryListView, PrefixListEntryEditView, \
    PrefixListEntryDeleteView, PrefixListEntryView, RouteMapEntryListView, RouteMapEntryView, RouteMapEntryEditView, \
    RouteMapEntryDeleteView

__all__ = (
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteEditView',
    'StaticRouteDeleteView',

    'PrefixListListView',
    'PrefixListView',
    'PrefixListEditView',
    'PrefixListDeleteView',
    'PrefixListEntryListView',
    'PrefixListEntryView',
    'PrefixListEntryEditView',
    'PrefixListEntryDeleteView',

    'RouteMapListView',
    'RouteMapView',
    'RouteMapEditView',
    'RouteMapDeleteView',
    'RouteMapEntryListView',
    'RouteMapEntryView',
    'RouteMapEntryEditView',
    'RouteMapEntryDeleteView'

)



from .static import StaticRouteFilterSet
from .objects import PrefixListFilterSet, PrefixListEntryFilterSet, RouteMapFilterSet, RouteMapEntryFilterSet

__all__ = (
    'StaticRouteFilterSet',

    'PrefixListFilterSet',
    'PrefixListEntryFilterSet',
    'RouteMapFilterSet',
    'RouteMapEntryFilterSet'
)



from .static import NestedStaticRouteSerializer
from .objects import NestedPrefixListSerializer, NestedPrefixListEntrySerializer, NestedRouteMapSerializer,\
    NestedRouteMapEntrySerializer

__all__ = (
    'NestedStaticRouteSerializer',

    'NestedPrefixListSerializer',
    'NestedPrefixListEntrySerializer',
    'NestedRouteMapSerializer',
    'NestedRouteMapEntrySerializer',
)



from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices

menu_items = (
    PluginMenuItem(
        link='plugins:netbox_routing:staticroute_list',
        link_text='Static Route',
        buttons=(
            PluginMenuButton('plugins:netbox_routing:staticroute_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
            PluginMenuButton('plugins:netbox_routing:staticroute_import', 'Import', 'mdi mdi-upload', ButtonColorChoices.CYAN),
        )
    ),
    PluginMenuItem(
        link='plugins:netbox_routing:prefixlist_list',
        link_text='Prefix Lists',
        buttons=(
            PluginMenuButton('plugins:netbox_routing:prefixlist_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
            PluginMenuButton('plugins:netbox_routing:prefixlist_import', 'Import', 'mdi mdi-upload', ButtonColorChoices.CYAN),
        )
    ),
    PluginMenuItem(
        link='plugins:netbox_routing:routemap_list',
        link_text='Route Maps',
        buttons=(
            PluginMenuButton('plugins:netbox_routing:routemap_add', 'Add', 'mdi mdi-plus', ButtonColorChoices.GREEN),
            PluginMenuButton('plugins:netbox_routing:routemap_import', 'Import', 'mdi mdi-upload', ButtonColorChoices.CYAN),
        )
    ),
)


from .static import StaticRouteSerializer
from .objects import PrefixListSerializer, PrefixListEntrySerializer, RouteMapSerializer, RouteMapEntrySerializer

__all__ = (
    'StaticRouteSerializer',

    'PrefixListSerializer',
    'PrefixListEntrySerializer',
    'RouteMapSerializer',
    'RouteMapEntrySerializer',
)



from django.urls import path

from netbox.views.generic import ObjectChangeLogView

from . import views
from .models import StaticRoute, PrefixList, PrefixListEntry, RouteMap, RouteMapEntry

urlpatterns = [
    path('routes/static/', views.StaticRouteListView.as_view(), name='staticroute_list'),
    path('routes/static/add/', views.StaticRouteEditView.as_view(), name='staticroute_add'),
    path('routes/static/import/', views.StaticRouteListView.as_view(), name='staticroute_import'),
    path('routes/static/<int:pk>/', views.StaticRouteView.as_view(), name='staticroute'),
    path('routes/static/<int:pk>/edit/', views.StaticRouteEditView.as_view(), name='staticroute_edit'),
    path('routes/static/<int:pk>/delete/', views.StaticRouteDeleteView.as_view(), name='staticroute_delete'),
    path('routes/static/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='staticroute_changelog', kwargs={'model': StaticRoute}),

    path('prefix-list/', views.PrefixListListView.as_view(), name='prefixlist_list'),
    path('prefix-list/add/', views.PrefixListEditView.as_view(), name='prefixlist_add'),
    path('prefix-list/import/', views.PrefixListListView.as_view(), name='prefixlist_import'),
    path('prefix-list/<int:pk>/', views.PrefixListView.as_view(), name='prefixlist'),
    path('prefix-list/<int:pk>/edit/', views.PrefixListView.as_view(), name='prefixlist_edit'),
    path('prefix-list/<int:pk>/delete/', views.PrefixListView.as_view(), name='prefixlist_delete'),
    path('prefix-list/<int:pk>/changelog/', views.PrefixListView.as_view(), name='prefixlist_changelog', kwargs={'model': PrefixList}),

    path('prefix-list-entry/', views.PrefixListEntryListView.as_view(), name='prefixlistentry_list'),
    path('prefix-list-entry/add/', views.PrefixListEntryEditView.as_view(), name='prefixlistentry_add'),
    path('prefix-list-entry/import/', views.PrefixListEntryListView.as_view(), name='prefixlistentry_import'),
    path('prefix-list-entry/<int:pk>/', views.PrefixListEntryView.as_view(), name='prefixlistentry'),
    path('prefix-list-entry/<int:pk>/edit/', views.PrefixListEntryEditView.as_view(), name='prefixlistentry_edit'),
    path('prefix-list-entry/<int:pk>/delete/', views.PrefixListEntryDeleteView.as_view(), name='prefixlistentry_delete'),
    path('prefix-list-entry/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='prefixlistentry_changelog', kwargs={'model': PrefixListEntry}),

    path('route-map/', views.RouteMapListView.as_view(), name='routemap_list'),
    path('route-map/add/', views.RouteMapEditView.as_view(), name='routemap_add'),
    path('route-map/import/', views.RouteMapListView.as_view(), name='routemap_import'),
    path('route-map/<int:pk>/', views.RouteMapView.as_view(), name='routemap'),
    path('route-map/<int:pk>/edit/', views.RouteMapView.as_view(), name='routemap_edit'),
    path('route-map/<int:pk>/delete/', views.RouteMapView.as_view(), name='routemap_delete'),
    path('route-map/<int:pk>/changelog/', views.RouteMapView.as_view(), name='routemap_changelog', kwargs={'model': RouteMap}),

    path('route-map-entry/', views.RouteMapEntryListView.as_view(), name='routemapentry_list'),
    path('route-map-entry/add/', views.RouteMapEntryEditView.as_view(), name='routemapentry_add'),
    path('route-map-entry/import/', views.RouteMapEntryListView.as_view(), name='routemapentry_import'),
    path('route-map-entry/<int:pk>/', views.RouteMapEntryView.as_view(), name='routemapentry'),
    path('route-map-entry/<int:pk>/edit/', views.RouteMapEntryEditView.as_view(), name='routemapentry_edit'),
    path('route-map-entry/<int:pk>/delete/', views.RouteMapEntryDeleteView.as_view(), name='routemapentry_delete'),
    path('route-map-entry/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='routemapentry_changelog', kwargs={'model': RouteMapEntry}),
]



from extras.plugins import PluginConfig

try:
    from importlib.metadata import metadata
except ModuleNotFoundError:
    from importlib_metadata import metadata

plugin = metadata('netbox_routing')


class NetboxRouting(PluginConfig):
    name = plugin.get('Name').replace('-', '_')
    verbose_name = plugin.get('Summary')
    description = plugin.get('Description')
    version = plugin.get('Version')
    author = plugin.get('Author')
    author_email = plugin.get('Author-email')
    base_url = 'routing'
    min_version = '3.2.0b1'
    required_settings = []
    caching_config = {}
    default_settings = {}


config = NetboxRouting



from netbox.tables import NetBoxTable
from netbox_routing.models import PrefixList, PrefixListEntry, RouteMap, RouteMapEntry


class PrefixListTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = PrefixList
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class PrefixListEntryTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = PrefixListEntry
        fields = ('pk', 'id', 'prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')
        default_columns = ('pk', 'id', 'prefix_list', 'sequence', 'type', 'prefix', 'le', 'ge')


class RouteMapTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = RouteMap
        fields = ('pk', 'id', 'name')
        default_columns = ('pk', 'id', 'name')


class RouteMapEntryTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = RouteMapEntry
        fields = ('pk', 'id', 'route_map', 'sequence', 'type')
        default_columns = ('pk', 'id', 'route_map', 'sequence', 'type')
